@@ -14,8 +14,8 @@
 
 #ifndef COMPUTEDOTPRODUCT_HPP
 #define COMPUTEDOTPRODUCT_HPP
-#include "Geometry.hpp"
-int ComputeDotProduct(const local_int_t n, const double * const x, const double * const y,
-    double * const result, double & time_allreduce, bool & isOptimized);
+#include "Vector.hpp"
+int ComputeDotProduct(const local_int_t n, const Vector & x, const Vector & y,
+    double & result, double & time_allreduce, bool & isOptimized);
 
 #endif // COMPUTEDOTPRODUCT_HPP
