@@ -20,9 +20,14 @@
 
 #include "ComputeSPMV_ref.hpp"
 
+#ifndef HPCG_NOMPI
+#include "ExchangeHalo.hpp"
+#endif
+
 #ifndef HPCG_NOOPENMP
 #include <omp.h>
 #endif
+#include <cassert>
 
 /*!
   Routine to compute matrix vector product y = Ax where:
@@ -39,8 +44,16 @@
 
   @see ComputeSPMV
 */
-int ComputeSPMV_ref( const SparseMatrix & A, const double * const x, double * const y) {
+int ComputeSPMV_ref( const SparseMatrix & A, Vector & x, Vector & y) {
 
+  assert(x.localLength>=A.localNumberOfColumns); // Test vector lengths
+  assert(y.localLength>=A.localNumberOfRows);
+
+#ifndef HPCG_NOMPI
+    ExchangeHalo(A,x);
+#endif
+  const double * const xv = x.values;
+  double * const yv = y.values;
   const local_int_t nrow = A.localNumberOfRows;
 #ifndef HPCG_NOOPENMP
   #pragma omp parallel for
@@ -52,8 +65,8 @@ int ComputeSPMV_ref( const SparseMatrix & A, const double * const x, double * co
     const int cur_nnz = A.nonzerosInRow[i];
 
     for (int j=0; j< cur_nnz; j++)
-      sum += cur_vals[j]*x[cur_inds[j]];
-    y[i] = sum;
+      sum += cur_vals[j]*xv[cur_inds[j]];
+    yv[i] = sum;
   }
   return(0);
 }
