@@ -18,8 +18,11 @@
  HPCG routine
  */
 
-#include "Geometry.hpp"
+#ifndef HPCG_NOMPI
+#include "ExchangeHalo.hpp"
+#endif
 #include "ComputeSYMGS_ref.hpp"
+#include <cassert>
 
 /*!
   Computes one step of symmetric Gauss-Seidel:
@@ -33,36 +36,49 @@
 
   Symmetric Gauss-Seidel notes:
   - We use the input vector x as the RHS and start with an initial guess for y of all zeros.
-  - We perform one forward sweep.  Since y is initially zero we can ignore the upper triangular terms of A.
+  - We perform one forward sweep.  x should be initially zero on the first GS sweep, but we do not attempt to exploit this fact.
   - We then perform one back sweep.
-       - For simplicity we include the diagonal contribution in the for-j loop, then correct the sum after
+  - For simplicity we include the diagonal contribution in the for-j loop, then correct the sum after
 
   @param[in] A the known system matrix
-  @param[in] x the input vector
-  @param[in] y On exit contains the result of one symmetric GS sweep with x as the RHS.
+  @param[in] r the input vector
+  @param[inout] x On entry, x should contain relevant values, on exit x contains the result of one symmetric GS sweep with r as the RHS.
+
+
+  @warning Early versions of this kernel (Version 1.1 and earlier) had the r and x arguments in reverse order, and out of sync with other kernels.
 
   @return returns 0 upon success and non-zero otherwise
 
   @see ComputeSYMGS
 */
-int ComputeSYMGS_ref( const SparseMatrix & A, const double * const x, double * const y) {
+int ComputeSYMGS_ref( const SparseMatrix & A, const Vector & r, Vector & x) {
+
+  assert(x.localLength==A.localNumberOfColumns); // Make sure x contain space for halo values
+
+#ifndef HPCG_NOMPI
+  ExchangeHalo(A,x);
+#endif
 
   const local_int_t nrow = A.localNumberOfRows;
   double ** matrixDiagonal = A.matrixDiagonal;  // An array of pointers to the diagonal entries A.matrixValues
+  const double * const rv = r.values;
+  double * const xv = x.values;
 
   for (local_int_t i=0; i< nrow; i++) {
     const double * const currentValues = A.matrixValues[i];
     const local_int_t * const currentColIndices = A.mtxIndL[i];
-    const double * addressOfCurrentDiagonal = matrixDiagonal[i]; // Current diagonal value
-    const int currentNumberOfNonzeros = addressOfCurrentDiagonal - currentValues;
-    double sum = x[i]; // RHS value
+    const int currentNumberOfNonzeros = A.nonzerosInRow[i];
+    const double  currentDiagonal = matrixDiagonal[i][0]; // Current diagonal value
+    double sum = rv[i]; // RHS value
 
     for (int j=0; j< currentNumberOfNonzeros; j++) {
       local_int_t curCol = currentColIndices[j];
-      if (curCol<nrow) sum -= currentValues[j] * y[curCol];
+      sum -= currentValues[j] * xv[curCol];
     }
+    sum += xv[i]*currentDiagonal; // Remove diagonal contribution from previous loop
+
+    xv[i] = sum/currentDiagonal;
 
-    y[i] = sum/(*addressOfCurrentDiagonal);
   }
 
   // Now the back sweep.
@@ -72,15 +88,15 @@ int ComputeSYMGS_ref( const SparseMatrix & A, const double * const x, double * c
     const local_int_t * const currentColIndices = A.mtxIndL[i];
     const int currentNumberOfNonzeros = A.nonzerosInRow[i];
     const double  currentDiagonal = matrixDiagonal[i][0]; // Current diagonal value
-    double sum = x[i]; // RHS value
+    double sum = rv[i]; // RHS value
 
     for (int j = 0; j< currentNumberOfNonzeros; j++) {
       local_int_t curCol = currentColIndices[j];
-      if (curCol<nrow) sum -= currentValues[j]*y[curCol];
+      sum -= currentValues[j]*xv[curCol];
     }
-    sum += y[i]*currentDiagonal; // Remove diagonal contribution from previous loop
+    sum += xv[i]*currentDiagonal; // Remove diagonal contribution from previous loop
 
-    y[i] = sum/currentDiagonal;
+    xv[i] = sum/currentDiagonal;
   }
 
   return(0);
