@@ -22,7 +22,7 @@
 #ifndef HPCG_NOOPENMP
 #include <omp.h>
 #endif
-
+#include <cassert>
 /*!
   Routine to compute the update of a vector with the sum of two
   scaled vectors where: w = alpha*x + beta*y
@@ -39,24 +39,30 @@
 
   @see ComputeWAXPBY
 */
-int ComputeWAXPBY_ref(const local_int_t n, const double alpha, const double * const x,
-    const double beta, const double * const y,
-    double * const w) {
+int ComputeWAXPBY_ref(const local_int_t n, const double alpha, const Vector & x,
+    const double beta, const Vector & y, Vector & w) {
+
+	assert(x.localLength>=n); // Test vector lengths
+	assert(y.localLength>=n);
+
+	const double * const xv = x.values;
+	const double * const yv = y.values;
+	double * const wv = w.values;
   if (alpha==1.0) {
 #ifndef HPCG_NOOPENMP
     #pragma omp parallel for
 #endif
-    for (local_int_t i=0; i<n; i++) w[i] = x[i] + beta * y[i];
+    for (local_int_t i=0; i<n; i++) wv[i] = xv[i] + beta * yv[i];
   } else if (beta==1.0) {
 #ifndef HPCG_NOOPENMP
     #pragma omp parallel for
 #endif
-    for (local_int_t i=0; i<n; i++) w[i] = alpha * x[i] + y[i];
+    for (local_int_t i=0; i<n; i++) wv[i] = alpha * xv[i] + yv[i];
   } else  {
 #ifndef HPCG_NOOPENMP
     #pragma omp parallel for
 #endif
-    for (local_int_t i=0; i<n; i++) w[i] = alpha * x[i] + beta * y[i];
+    for (local_int_t i=0; i<n; i++) wv[i] = alpha * xv[i] + beta * yv[i];
   }
 
   return(0);
