@@ -58,7 +58,7 @@ string YAML_Doc::generateYAML() {
   ptm = localtime(&rawtime);
   char sdate[25];
   //use tm_mon+1 because tm_mon is 0 .. 11 instead of 1 .. 12
-  sprintf (sdate,"%04d:%02d:%02d-%02d:%02d:%02d",ptm->tm_year + 1900, ptm->tm_mon+1,
+  sprintf (sdate,"%04d.%02d.%02d.%02d.%02d.%02d",ptm->tm_year + 1900, ptm->tm_mon+1,
       ptm->tm_mday, ptm->tm_hour, ptm->tm_min,ptm->tm_sec);
 
   string filename;
@@ -69,11 +69,7 @@ string YAML_Doc::generateYAML() {
   filename = filename + string(sdate) + ".yaml";
   if (destinationDirectory!="" && destinationDirectory!=".") {
     string mkdir_cmd = "mkdir " + destinationDirectory;
-#ifdef REDSTORM
-    mkdir(destinationDirectory.c_str(),0755);
-#else
     system(mkdir_cmd.c_str());
-#endif
     filename = destinationDirectory + "/" + destinationFileName;
   } else
     filename = "./" + filename;
